// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

package qtprotobufnamespace.tests;  // Generated namespaces will be prepended with
                                    // the extra namespace
message EmptyMessage {}

message SimpleStringMessage {
    string testFieldString = 6;
}

message ComplexMessage {
    int32 testFieldInt = 1;
    SimpleStringMessage testComplexField = 2;
}
