// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

package qtprotobufnamespace.qttypes.tests;
import "QtCore/QtCore.proto";

message NestedQUrlMessage {
    message QUrlMessage {
        QtCore.QUrl testField = 1;
    }
    NestedQUrlMessage.QUrlMessage testField = 1;
}
