// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

syntax = "proto3";
package qt.protobuf;

message MessageType {
}

message MessageHoder {
    MessageType mutField = 1;
    MessageType field = 2;
}
