// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

package qtgrpc.tests;

message SimpleStringMessage {
    string testFieldString = 6;
}

message SimpleIntMessage {
    sint32 testField = 1;
}

message BlobMessage {
    bytes testBytes = 1;
}

service TestService {
    rpc testMethod(SimpleStringMessage) returns (SimpleStringMessage) {}
    rpc testMethodServerStream(SimpleStringMessage) returns (stream SimpleStringMessage) {}
    rpc testMethodClientStream(stream SimpleStringMessage) returns (SimpleStringMessage) {}
    rpc testMethodBiStream(stream SimpleStringMessage) returns (stream SimpleStringMessage) {}
}
