/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */

#if defined(QT_SHARED) || !defined(QT_STATIC)
#  if defined(QT_BUILD_TST_QTPROTOBUFGEN_QML_URI_LIB)
#    define QPB_TST_QTPROTOBUFGEN_QML_URI_EXPORT Q_DECL_EXPORT
#  else
#    define QPB_TST_QTPROTOBUFGEN_QML_URI_EXPORT Q_DECL_IMPORT
#  endif
#else
#  define QPB_TST_QTPROTOBUFGEN_QML_URI_EXPORT
#endif

